
USE AdventureWorksDW2008Big;
GO

-- metadata
select * from sys.indexes
where name = 'csindx_FactResellerSalesPart';

-- one row per column in index, per partition
select * from sys.column_store_dictionaries;
-- counts, min/max, size
select * from sys.column_store_segments
where column_id = 2

-- An aggregate query joining two tables
-- Consider a star-join query that calculates how many of product 215 we sold each quarter. 
-- The fact table named FactResellerSalesPart is partitioned on the OrderDateKey column. The dimension table named DimDate links to the fact table through a primary key  foreign key relationship on the date key.
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;
GO

-- segment elimination?? - no
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215 AND f.OrderDateKey >= 20030101 AND f.OrderDateKey < 20040101
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;

-- Additionally, there will be queries that may only target a single table. In such cases, SQL Server tries to leverage the power of batch execution and column store technology to speed up query execution as well.

-- A simple aggregate query on a single table
-- The fact table is FactResellerSalesPart and this table is partitioned on the OrderDateKey column. The following query returns the number of rows and the number of orders.
-- Uses row mode
SELECT COUNT(*) AS NumberOfRows, 
    COUNT(DISTINCT(f.SalesOrderNumber)) AS NumberOfOrders
FROM dbo.FactResellerSalesPart AS f;
GO

-- Determine Size
SELECT SUM(on_disk_size_MB) AS TotalSizeInMB
FROM
(
   (SELECT SUM(css.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_segments AS css
        ON css.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPart') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
  UNION ALL
   (SELECT SUM(csd.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_dictionaries AS csd
        ON csd.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPart') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
) AS SegmentsPlusDictionary


-- Hint
SELECT d.CalendarYear,
    d.CalendarQuarter,
    COUNT(*) AS NumberSold
FROM dbo.FactResellerSalesPart AS f WITH (INDEX(csindx_FactResellerSalesPtnd))
    JOIN dbo.DimDate AS d
    ON f.OrderDateKey = d.DateKey
WHERE ProductKey = 215
GROUP BY d.CalendarYear, d.CalendarQuarter
ORDER BY d.CalendarYear, d.CalendarQuarter;

DBCC FREEPROCCACHE
GO

-- uses batch mode
SELECT SalesTerritoryKey, SUM(ExtendedAmount) AS SalesByTerritory
FROM FactResellerSalesPart
GROUP BY SalesTerritoryKey;
GO

-- look for new iterator
select top 1000 * 
into FactResellerSalesPartSubset 
from dbo.FactResellerSalesPart; 


-- Batch Hash Table Build iterator in Batch mode
select p.DueDateKey, count(*) 
from dbo.FactResellerSalesPart p, dbo.FactResellerSalesPartSubset m 
where p.SalesOrderLineNumber = m.SalesOrderLineNumber 
group by p.DueDateKey 
order by p.DueDateKey; 

-- new hint
SELECT SalesTerritoryKey, SUM(ExtendedAmount) AS SalesByTerritory
FROM FactResellerSalesPart
GROUP BY SalesTerritoryKey
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- does not use batch mode
select p.DueDateKey, count(*) as DueDateCount
from dbo.FactResellerSalesPartSubset m 
left outer join dbo.FactResellerSalesPart p
on p.SalesOrderLineNumber = m.SalesOrderLineNumber 
group by p.DueDateKey 
order by p.DueDateKey; 
GO